import config
from   OpenGL.GL import *
from   OpenGL.GLU import *
import math
import random as rand

import Geometry

W_ANY = 0
W_MAIN = 1
W_SIDE = 2

COLL_DIST = 0.5 / 50.0

class Chain:

   def __init__(self):
      self.head = None
   
   def detach(self):
      for ce in self.head.children:   
         ce.parent = None
      self.head.children = []
      self.head.ce_rots = []

      self.head.tx = self.head.px
      self.head.ty = self.head.py

   def isHeadColl(self, ch = None, ring = None):
      x, y = self.head.px, self.head.py
      if ch:
         cx, cy = ch.head.px, ch.head.py
      elif ring:
         cx, cy = ring

      dx = x - cx
      dy = y - cy

      dist2 = dx*dx + dy*dy
      return dist2 < COLL_DIST
         

   def initHead(self, pos, sizeMul=1.0):
      #tetra = Geometry.Tetrahedron(0.04*sizeMul, (1.0, 0.0, 0.0, 1.0))
      tetra = Geometry.GameObject("tetra", 0.04*sizeMul, (0.0, 0.0, 1.0, 1.0), (0.0, 1.0, 0.0, 1.0))
      self.head = ChainElem(tetra, pos)
  
   def addElems(self, num):
      for i in xrange(num):
         col_mul, col = rand.sample([
            (1, (0.0, 1.0, 0.0, 1.0)),
            (4, (1.0, 0.0, 0.0, 1.0))
         ], 1)[0]
         
         mem, geom = rand.sample([
            # Geometry.GameObject("tetra", 0.03, (0.8, 0.1, 0.0, 1.0)),
            (20, Geometry.GameObject("cube", 0.03, col)),
            (40, Geometry.GameObject("octa", 0.03, col))
         ], 1)[0]

         geom.mem = mem * col_mul
         self.addToEnd(ChainElem(geom))

   def getMainString(self):
      ret = [self.head]

      next = self.head.getNextMain()
      while next:
         ret.append(next)
         next = next.getNextMain()
        
      return ret
 
   def setEdgeCol(self, col):
      for e in self.getMainString():
         e.geom.edge_color = col

   def getEndElem(self, ce):
      last = ce
      next = ce.getNextMain()
      while next:
         last = next
      
      return last         

   def getMainElem(self, i):
      return self.getMainString()[i]

   def getSideElems(self, i):
      return self.getMainElem(i).getNextSides()

   def setHead(self, h):
      self.head = h
 
   def addChain(self, c, where=W_MAIN):
      try:
         e = c.getMainElem(1)
      except:
         return

      self.addElem(e)

   def addElem(self, e):
      if e is None:
         return
         
      # determine where to add

      self.addToEnd(e)
      return

      if rand.uniform(0.0, 1.0) < 0.2:
         self.addToEnd(e)
      else:
         str = self.getMainString()
         lenstr = len(str)
         ce = str[rand.randint(0, lenstr-1)]

         sides = ce.getNextSides()

         if len(sides) == 0:
            print 'No sides!', sides
            print 'ce rots', ce.ce_rots
            ce.addChild(e, 0.05)
            print 'e relang', e.rel_ang
            e.debug = 1
         else:
            print 'Sides', sides

  
   def addToEnd(self, e, rel_dist = 0.05):
      if self.head is None:
         self.head = e
      else:
         self.getMainElem(-1).addChild(e, rel_dist)
  
   def calcStep(self, dt):
      if self.head is not None:
         self.head.calcStep(dt)
   
   def backProp(self, dt):
      if self.head is not None:
         self.head.backProp(dt)
   
   def addRel(self, x, y):
      if self.head is not None:
         self.head.addRel(x,y)
 
   def draw(self):
      if self.head is not None:
         self.head.draw()
 
   def getMem(self):
      d_mem = 0
      for c in self.getMainString():
         try:
            d_mem += c.geom.mem
         except:
            # head elem
            pass
      return d_mem


class WanderChain(Chain):
   def __init__(self, badchain=0.2, speedmul=1.0):
      Chain.__init__(self)
      
      x,y = rand.uniform(config.SCREEN_LEFT, config.SCREEN_RIGHT), rand.uniform(-0.5, 0.5)
           
      self.initHead((x, y))
      self.addElems(rand.randint(1, 4))

      self.bad = rand.uniform(0.0, 1.0) < badchain
      if self.bad:
         for ce in self.getMainString():
            r,g,b,a = map(lambda x:max(0.1, x*0.2), ce.geom.face_color) #(0.4, 0.4, 0.4, 1.0)
            ce.geom.face_color = (r,g,b,1.0)

      self.vx = rand.uniform(-13.0, 13.0) / 100.0 * speedmul
      self.vy = rand.uniform(-10.0, 10.0) / 100.0 * speedmul

      if x > 0.0 and self.vx > 0.0: self.vx *= -1
      if x < 0.0 and self.vx < 0.0: self.vx *= -1
      if y > 0.0 and self.vy > 0.0: self.vy *= -1
      if y < 0.0 and self.vy < 0.0: self.vy *= -1

      self.size = 0.1

      self.sumt = 0.0

      self.circ = rand.uniform(0.0, 1.0) < 0.3

      self.dying = False
      self.dead = 0.0

      self.xphase = rand.uniform(0.0, 2*math.pi)
      self.yphase = rand.uniform(0.0, 2*math.pi)

   def markDead(self):
      self.dying = True
      self.dead = 1.0

      for ce in self.getMainString():
         r,g,b,a = ce.geom.face_color
         ce.geom.face_color = r,g,b,0.3

   def update(self, dt):
      self.sumt += dt

      self.size = min(1.0, self.size + dt)

      if self.dying:
         self.dead -= dt
         return
      
      dx, dy = self.vx*dt, self.vy*dt
      
      if self.circ:
         dx += math.cos(self.sumt + self.xphase) / 100.0
         dy += math.sin(self.sumt + self.yphase) / 100.0

      self.addRel(dx,dy)

   def draw(self):
      if self.head is not None:
         self.head.draw(self.size)

class CircleWanderChain(Chain):
   def __init__(self, pos, radius, speed):
      Chain.__init__(self)

      self.radius = radius
      self.speed = speed
      self.ang = 0.0
      
      x,y = pos
           
      self.initHead((x, y))
      self.addElems(rand.randint(0, 5))

   def update(self, elapsed_ms):
      dt = elapsed_ms / 1000.0
      self.ang += self.speed * dt
      self.addRel(self.radius * math.cos(self.ang), self.radius * math.sin(self.ang))


class ChainElem:

   def __init__(self, geom, pos=(0.0, 0.0), parent=None):
      self.geom = geom

      self.parent = parent
      self.children = []

      self.px, self.py = pos
      self.tx, self.ty = map(lambda x:x+rand.uniform(-0.05, 0.05), pos)

      self.angle = 0.0
      self.t_ang = 0.0

      self.rel_dist = 0.0
      self.rel_ang = 0.0

      self.ce_rots = []

      self.ease_param = 0.8

      self.amb_rotation = 90.0
      
      self._sum_amb_rot = rand.uniform(1.0, 10.0) * 20.0
      self.amb_rot_speed = rand.uniform(1.0, 2.5)

   def setColor(self, color):
      self.geom.face_color = color

   def getNextMain(self):
      if len(self.children):
         ce = self.children[0]
         if ce.rel_ang == 0.0:
            return ce

      return None
 
   def getNextSides(self):
      return filter(
         lambda x:x.rel_ang != 0.0,
         self.children
      )

   def setTemporalDamping(self, d):
      self.temp_damp = d

   def setAccel(self, acc):
      self.accel = acc

   def draw(self, size=1.0):
      try:
         if self.debug:
            print 'DBG', self.angle, self.rel_ang, self.t_ang
      except:
         pass
      
      glPushMatrix()
      #glLoadIdentity()
      glTranslatef(self.px, self.py, 0.0)
      glScalef(size, size, size)
      glRotatef(90.0, 0.0, 1.0, 0.0)
      glRotatef(self.angle, -1.0, 0.0, 0.0)
      glRotatef(self._sum_amb_rot * self.amb_rot_speed, 0.0, 0.0, 1.0)
      self.geom.draw()
      glPopMatrix()

      for ce in self.children:
         ce.draw(size)
      
   def addChild(self, child, rel_dist = 0.1, rel_ang = None):
      if rel_ang is None:
         if len(self.ce_rots) == 0:
            rel_ang = 0.0
         else:
            a = abs(self.ce_rots[0])
            b = abs(self.ce_rots[-1])

            if a < b:
               rel_ang = self.ce_rots[0] - config.ANG_STEP
            else:
               rel_ang = self.ce_rots[-1] + config.ANG_STEP

      self.ce_rots.append(rel_ang)
      self.ce_rots.sort()

      child.rel_dist = rel_dist
      child.rel_ang = rel_ang + self.rel_ang

      child.parent = self
      self.children.append(child)
   
   def addRel(self, x, y):
      self.tx += x
      self.ty += y

   def backProp(self, dt):

      # Apply forces to/from children
      for ce in self.children:

         vec_x = ce.px- self.px
         vec_y = ce.py - self.py

         dist = math.sqrt(vec_x*vec_x + vec_y*vec_y)

         ce.backProp(dt)

   def calcStep(self, dt):

      # ambient rotation

      self._sum_amb_rot += self.amb_rotation * dt

      # real thing

      if self.parent is not None:
         # calculate target position
         rad = (self.parent.angle  + 180.0 + self.rel_ang) / 180.0 * math.pi
         
         dx = self.rel_dist * math.cos(rad)
         dy = self.rel_dist * math.sin(rad)

         self.tx = self.parent.px + dx
         self.ty = self.parent.py + dy

      ea = self.ease_param 

      self.px = ea*self.px + (1.0-ea)*self.tx;
      self.py = ea*self.py + (1.0-ea)*self.ty;

      dx = self.tx - self.px
      dy = self.ty - self.py

      if math.sqrt(dx*dx + dy*dy) < 0.001:
         dx, dy = 0.0, 0.0
      else:
         self.t_ang = math.atan2(dy,dx) / math.pi * 180.0

      if abs(self.angle-(self.t_ang+360.0)) < abs(self.angle-self.t_ang):
         self.t_ang += 360.0
      if abs(self.angle-(self.t_ang-360.0)) < abs(self.angle-self.t_ang):
         self.t_ang -= 360.0

      ea = 0.1

      if abs(self.angle-self.t_ang) > 1.0:
         self.angle = ea*self.angle + (1.0-ea)*self.t_ang


      for ce in self.children:
         ce.calcStep(dt)
      

         
       

