import math

import pygame
from   pygame.locals import *

from   OpenGL.GL import *
from   OpenGL.GLU import *

import config

import Texture, Scene, Geometry

class Menu:
   SEL_SCALE = 1.1
   def __init__(self, menu_pts, font_name, font_size, color, sel_color, ypos, width, height):
      self.menu_pts = menu_pts
      self.ypos = ypos
      self.width = width
      self.height = height
      self.color = color
      self.sel_color = sel_color
      self.act = 0
      self.sel_cb = None

      self._createTexts(font_name, font_size)
      self._calcPositions()

      self.texts[self.act][0].scale(Menu.SEL_SCALE)

   def _createTexts(self, font_name, font_size):
      self.texts = []
      for menu_text in self.menu_pts:
         self.texts.append( [ Texture.Text(menu_text, font_name, font_size), 0.0 ] )

   def _calcPositions(self):
      n = len(self.menu_pts) + 1
      i = 1
      for t in self.texts:
         w, h = t[0].getSize()
         t[1] = self.ypos + self.height / 2.0 - float(self.height) * i / n
         i += 1

   def _getMousePt(self, mx, my):
      i = 0
      for text, ypos in self.texts:
         w, h = text.getSize()

         if mx >= -w / 2.0 and mx <= w / 2.0 and my >= ypos - h / 2.0 and my <= ypos + h / 2.0:
            return i

         i += 1

      return -1

   def onKeyEvent(self, e):
      if e.key == K_UP:
         if self.act == 0:
            self.setSelected(len(self.texts) - 1)
         else:
            self.setSelected(self.act - 1)

      elif e.key == K_DOWN:
         if self.act == len(self.texts) - 1:
            self.setSelected(0)
         else:
            self.setSelected(self.act + 1)

      elif e.key == K_RETURN:
         if self.sel_cb is not None:
            config.sound_manager["select.ogg"].play()
            self.sel_cb()

   def onMouseEvent(self, e):
      mx, my = (float(e.pos[0]) / config.VIDEO_MODE[0] - 0.5) * config.ASPECT_RATIO, 0.5 - float(e.pos[1]) / config.VIDEO_MODE[1]
      
      if e.type == MOUSEMOTION:
         menu_pt = self._getMousePt(mx, my)
         if menu_pt != -1 and self.act != menu_pt:
            self.setSelected(menu_pt)
            
      elif e.type == MOUSEBUTTONDOWN:
         menu_pt = self._getMousePt(mx, my)
         if menu_pt != -1 and self.sel_cb is not None:
            config.sound_manager["select.ogg"].play()
            self.sel_cb()

   def setSelectCallback(self, sel_cb):
      self.sel_cb = sel_cb

   def getSelected(self):
      return self.act

   def setSelected(self, sel):
      config.sound_manager["switch.ogg"].play()
      #self.texts[self.act][0].scale(1.0 / Menu.SEL_SCALE)
      self.act = sel
      #self.texts[self.act][0].scale(Menu.SEL_SCALE)

   def draw(self):
      glPushMatrix()      
      glPushAttrib(GL_CURRENT_BIT)
      
      prev_vpos = 0.0
      i = 0
      for text, vpos in self.texts:
         glTranslatef(0.0, vpos - prev_vpos, 0.0)
         prev_vpos = vpos
         
         glColor4f(*self.color)
         text.draw()
         
         if i == self.act:
            glColor4f(*self.sel_color)
            text.scale(Menu.SEL_SCALE)
            text.draw()
            text.scale(1.0 / Menu.SEL_SCALE)
         i += 1

      glPopAttrib()
      glPopMatrix()

